/*
 * timer_sub.c
 *
 *  Created on: 2021/03/23
 *      Author: 0170886
 */
#include <kernel.h>
#include "kernel_cfg.h"
#include <t_syslog.h>
#include <t_stdlib.h>

#include "device.h"
#include "limits.h"
#include "cup_timer.h"
#include "common.h"
#include "timer.h"

/*
 *  Glboal Variable
 */
extern TTM_CTL	gtm_ctl;

/* システム時刻による時間の測定(デバッグ用) */
static SYSUTM gStartTime;
static SYSUTM gCurrentTime;

/*
 *  Timer Task
 */
void
StOffEntry(void){
	DEBUG_SYSLOG(LOG_NOTICE, " Timer Off Entry!");
}

void
StOnEntry(void){
	DEBUG_SYSLOG(LOG_NOTICE, " Timer On Entry!");
	GTM_REMTM = INIT_TIME;		/* タイムアウト時間を初期値に設定 	*/

	SVC_PERROR(get_utm(&gStartTime)); /* スタート時刻を記憶 */

	sta_cyc(BASE_TIME_HANDLER);
}

void
StOffEvSw1Off(void){
	/* メモリプール解放禁止を設定 */
	GTM_CTLF |= DTM_NOTREL;

	/* 受信したメッセージのイベントを書き換えて転送 */
	GTM_RCVEV =  EV_BLINK_OFF;
    DEBUG_SYSLOG(LOG_NOTICE, " snd_dtq = 0x%p", GTM_RCVAD);
	SVC_PERROR(snd_dtq(BLINK_DTQ, (intptr_t)GTM_RCVAD));
	DEBUG_SYSLOG(LOG_NOTICE, " Timer stop!");
}

void
StOnEvSw1Off(void){
	/* 状態遷移有り,メモリプール解放禁止を設定 */
	GTM_CTLF |= (DTM_STSCHG | DTM_NOTREL);

	GTM_NEXTSTS = ST_TIMER_OFF;	/* 次状態を「タイマOFF」状態に設定 	*/

	/* 受信したメッセージのイベントを書き換えて転送 */
	GTM_RCVEV = EV_BLINK_OFF;

    DEBUG_SYSLOG(LOG_NOTICE, " snd_dtq = 0x%p", GTM_RCVAD);
	SVC_PERROR(snd_dtq(BLINK_DTQ, (intptr_t)GTM_RCVAD));
	DEBUG_SYSLOG(LOG_NOTICE, " Timer stop!");
//	GTM_REMTM = 0;
//	stp_cyc(BASE_TIME_HANDLER);
}

void
StOffEvSw1On(void){
	/* 状態遷移有り,メモリプール解放禁止を設定 */
	GTM_CTLF |= (DTM_STSCHG | DTM_NOTREL);

	GTM_NEXTSTS = ST_TIMER_ON;	/* 次状態を「タイマON」状態に設定 	*/

//	GTM_REMTM = INIT_TIME;		/* タイムアウト時間を初期値に設定 	*/

	/* 受信したメッセージのイベントを書き換えて転送 */
	GTM_RCVEV = EV_BLINK_ACTIVE;

    DEBUG_SYSLOG(LOG_NOTICE, " snd_dtq = 0x%p", GTM_RCVAD);
	SVC_PERROR(snd_dtq(BLINK_DTQ, (intptr_t)GTM_RCVAD));
	DEBUG_SYSLOG(LOG_NOTICE, " Timer start!");
//	sta_cyc(BASE_TIME_HANDLER);
}

void
StOnEvPush1On(void){
	GTM_REMTM = GTM_REMTM + EXTRA_UNIT;
	DEBUG_SYSLOG(LOG_NOTICE, " Extend %d sec. Remaining  %d sec",
	EXTRA_UNIT, GTM_REMTM);
}

void
StOnEvBaseTime(void){
	GTM_REMTM = GTM_REMTM - 1;
	if (GTM_REMTM == 0) {
		SVC_PERROR(get_utm(&gCurrentTime));
		DEBUG_SYSLOG(LOG_NOTICE, " System measurement time : %lu%s", gCurrentTime - gStartTime,"us");

		/* 状態遷移有り,メモリプール解放禁止を設定 */
		GTM_CTLF |= (DTM_STSCHG | DTM_NOTREL);
		GTM_NEXTSTS = ST_TIMER_OFF;	/* 次状態を「タイマOFF」状態に設定 	*/

		/* 受信したメッセージのイベントを書き換えて転送 */
		GTM_RCVEV = EV_BLINK_TIMEOUT;

        DEBUG_SYSLOG(LOG_NOTICE, " snd_dtq = 0x%p", GTM_RCVAD);
		SVC_PERROR(snd_dtq(BLINK_DTQ, (intptr_t)GTM_RCVAD));
		DEBUG_SYSLOG(LOG_NOTICE, " Timer timeout");
//		stp_cyc(BASE_TIME_HANDLER);
	} else if ((GTM_REMTM % ACT_INTERVAL) == 0) {
		SVC_PERROR(get_utm(&gCurrentTime));
		DEBUG_SYSLOG(LOG_NOTICE, " System measurement time : %lu%s", gCurrentTime - gStartTime,"us");

		/* メモリプール解放禁止を設定 */
		GTM_CTLF |= DTM_NOTREL;

		/* 受信したメッセージのイベントを書き換えて転送 */
		GTM_RCVEV = EV_BLINK_ACTIVE;

        DEBUG_SYSLOG(LOG_NOTICE, " snd_dtq = 0x%p", GTM_RCVAD);
		SVC_PERROR(snd_dtq(BLINK_DTQ, (intptr_t)GTM_RCVAD));
    	DEBUG_SYSLOG(LOG_NOTICE, " Remaining  %d sec", GTM_REMTM);
    }
}

void
StOffExit(void){
	DEBUG_SYSLOG(LOG_NOTICE, " Timer Off Exit!");
}

void
StOnExit(void){
	DEBUG_SYSLOG(LOG_NOTICE, " Timer On Exit!");
	GTM_REMTM = 0;		/* タイムアウト時間をクリア 	*/

	stp_cyc(BASE_TIME_HANDLER);
}
